@@ -2,25 +2,21 @@ package Class::Accessor::TrackDirty;
 use 5.008_001;
 use strict;
 use warnings;
-use List::MoreUtils qw(any);
 use Storable qw(dclone freeze);
-our $VERSION = '0.110.1';
+our $VERSION = '0.01';
 
 our $RESERVED_FIELD = '_original';
 our $NEW = 'new';
 our $FROM_HASH = 'from_hash';
-our $RAW = 'raw';
 our $TO_HASH = 'to_hash';
 our $IS_MODIFIED = 'is_dirty';
-our $MODIFIED_FIELDS = 'dirty_fields';
-our $IS_NEW = 'is_new';
 our $REVERT = 'revert';
 
 {
     my %package_info;
     sub _package_info($) {
         my $package = shift;
-        $package_info{$package} ||= {tracked_fields => {}, fields => {}};
+        $package_info{$package} ||= {tracked_fields => [], fields => []};
     }
 }
 
@@ -82,7 +78,7 @@ sub _make_accessor($$) {
 sub _mk_tracked_accessors($@) {
     my $package = shift;
     _make_tracked_accessor $package => $_ for @_;
-    @{(_package_info $package)->{tracked_fields}}{@_} = (1,) x @_;
+    push @{_package_info($package)->{tracked_fields}}, @_;
 }
 
 sub _mk_helpers($) {
@@ -97,7 +93,7 @@ sub _mk_helpers($) {
         my %modified = ref $_[0] eq 'HASH' ? %{$_[0]} : @_;
 
         my %origin;
-        for my $name (keys %$tracked_fields) {
+        for my $name (@$tracked_fields) {
             $origin{$name} = delete $modified{$name} if exists $modified{$name};
         }
 
@@ -105,7 +101,7 @@ sub _mk_helpers($) {
         bless \%modified, $package;
     };
 
-    *{"$package\::$RAW"} = sub {
+    *{"$package\::$TO_HASH"} = sub {
         my ($self) = @_;
 
         my %hash = (
@@ -113,56 +109,37 @@ sub _mk_helpers($) {
                 # Don't store undefined values.
                 my $v = $self->$_;
                 defined $v ? ($_ => $v) : ();
-            } keys %$tracked_fields, keys %$fields),
+            } @$tracked_fields, @$fields),
         );
 
+        # rewrite the caller directly
+        $_[0] = (ref $self)->from_hash(\%hash);
+
         return \%hash;
     };
 
-    *{"$package\::$TO_HASH"} = sub {
-        my ($self) = @_;
-        my $raw = $self->$RAW;
-
-        # Move published data for cleaning.
-        $self->{$RESERVED_FIELD} ||= {};
-        $self->{$RESERVED_FIELD}{$_} = delete $self->{$_}
-                         for grep { exists $self->{$_} } keys %$tracked_fields;
-
-        return $raw;
-    };
-
     *{"$package\::$IS_MODIFIED"} = sub {
-        my ($self, $field) = @_;
-        return any { $self->$IS_MODIFIED($_) } keys %$tracked_fields
-                                                                 unless $field;
-
-        return unless $tracked_fields->{$field};
-        return defined $self->{$field} unless defined $self->{$RESERVED_FIELD};
-
-        exists $self->{$field} &&
-               _is_different $self->{$field}, $self->{$RESERVED_FIELD}{$field};
-    };
-
-    *{"$package\::$MODIFIED_FIELDS"} = sub {
         my $self = shift;
-        grep { $self->$IS_MODIFIED($_) } keys %$tracked_fields;
-    };
+        return 1 unless defined $self->{$RESERVED_FIELD};
 
-    *{"$package\::$IS_NEW"} = sub {
-        my $self = shift;
-        exists $self->{$RESERVED_FIELD} ? 0 : 1;
+        for (@$tracked_fields) {
+            return 1
+                if exists $self->{$_} &&
+                   _is_different $self->{$_}, $self->{$RESERVED_FIELD}{$_};
+        }
+        return;
     };
 
     *{"$package\::$REVERT"} = sub {
         my $self = shift;
-        delete $self->{$_} for keys %$tracked_fields;
+        delete $self->{$_} for @$tracked_fields;
     };
 }
 
 sub _mk_accessors($@) {
     my $package = shift;
     _make_accessor $package => $_ for @_;
-    @{(_package_info $package)->{fields}}{@_} = (1,) x @_;
+    push @{_package_info($package)->{fields}}, @_;
 }
 
 sub _mk_new($) {
@@ -206,8 +183,6 @@ sub mk_new_and_tracked_accessors {
 1;
 __END__
 
-=encoding utf-8
-
 =head1 NAME
 
 Class::Accessor::TrackDirty - Define simple entities stored in some places.
@@ -231,10 +206,6 @@ Class::Accessor::TrackDirty - Define simple entities stored in some places.
     $user->name('honma'); # I can't make up my mind...
     # ... blabla ...
 
-    # Check the status of fields if needed
-    $user->is_dirty('name') and warn "Did you change name?";
-    my @dirty_fields = $user->dirty_fields;
-
     # Store it only if $user was really modified.
     store_into_someplace($user->to_hash) if $user->is_dirty;
 
@@ -257,8 +228,8 @@ You can pass a hash-ref or hash-like list to C<<new>> method.
 
 =item C<< my $object = YourClass->new({name1 => "value1", ...}); >>
 
-The instance created by C<<new>> is regarded as `dirty' if it has some nonempty
-fields. It's because it hasn't been stored yet.
+The instance created by C<<new>> is regarded as `dirty' since it hasn't been
+stored yet.
 
 =back
 
@@ -271,24 +242,9 @@ Following helper methods will be created automatically.
 
 =item C<< $your_object->is_dirty; >>
 
-=item C<< $your_object->is_dirty("field_name"); >>
-
 Check that the instance is modified. If it's true, you should store this
 instance into some place through using C<<to_hash>> method.
 
-When you pass the name of a field, you can know if the field contains the same
-value as the stored object.
-
-=item C<< my @fields = $your_object->dirty_fields; >>
-
-Gets the name of all dirty fields of C<$your_object>.
-
-=item C<< $your_object->is_new; >>
-
-Checks if the instance might be in a storage. Returns false value when
-the instance comes from C<from_hash> method, or after you call
-C<to_hash> method.
-
 =item C<< my $hash_ref = $your_object->to_hash; >>
 
 Eject data from this instance as plain hash-ref format.
@@ -297,12 +253,6 @@ C<<$your_object>> is regarded as `clean' after calling this method.
 You'd better store C<<$hash_ref>> into some place ASAP. It's up to you how
 C<<$hash_ref>> should be serialized.
 
-=item C<< $your_object->raw; >>
-
-Retrieves the row data from the instance. The return value is the same as
-C<to_hash> method, but this method doesn't change the state of the
-instance.
-
 =item C<< my $object = YourClass->from_hash({name1 => "value1", ...}); >>
 
 Rebuild the instance from a hash-ref ejected by C<<to_hash>> method.
